@@ -10,8 +10,10 @@ from airflow_kubernetes_job_operator.utils import random_string
 from airflow_kubernetes_job_operator.collections import JobRunnerDeletePolicy, JobRunnerException
 from airflow_kubernetes_job_operator.config import SHOW_RUNNER_ID_IN_LOGS
 from airflow_kubernetes_job_operator.kube_api import (
+    EventHandler,
     KubeApiConfiguration,
     GetAPIVersions,
+    KubeLogApiEvent,
     KubeApiRestQuery,
     KubeApiRestClient,
     KubeResourceKind,
@@ -68,6 +70,8 @@ class JobRunner:
             name_postfix (str, optional): The postfix for all resource name. Defaults to None.
         """
 
+        logger = logger if logger is not None else kube_logger
+
         body = body if not isinstance(body, str) else list(yaml.safe_load_all(body))
         body = body if isinstance(body, list) else [body]
 
@@ -156,17 +160,14 @@ class JobRunner:
             "Cannot create a job without a template, 'spec.template' is missing or not a dictionary"
         )
 
-        assert isinstance(descriptor.metadata["template"].get("spec", None), dict), JobRunnerException(
+        assert isinstance(descriptor.spec.get("template", {}).get("spec", None), dict), JobRunnerException(
             "Cannot create a job without a template spec, 'spec.template.spec' is missing or not a dictionary"
         )
 
         descriptor.spec.setdefault("backoffLimit", 0)
-        descriptor.metadata.setdefault("finalizers", [])
+        descriptor.metadata.setdefault("finalizers", ["foregroundDeletion"])
         descriptor.spec["template"]["spec"].setdefault("restartPolicy", "Never")
 
-        if "foregroundDeletion" not in descriptor.metadata["finalizers"]:
-            descriptor.metadata["finalizers"].append("foregroundDeletion")
-
     @classmethod
     def custom_prepare_pod_kind(cls, body: dict):
         assert isinstance(body, dict), ValueError("Body must be a dictionary")
@@ -214,6 +215,7 @@ class JobRunner:
                 + "more information cab be found @ "
                 + "https://github.com/LamaAni/KubernetesJobOperator/docs/add_custom_kinds.md",
             )
+        kind = KubeResourceKind.get_kind(kind_name)
 
         descriptor = KubeResourceDescriptor(body)
         assert descriptor.spec is not None, ValueError("body['spec'] is not defined")
@@ -230,8 +232,8 @@ class JobRunner:
             },
         )
 
-        if kind_name in self.custom_prepare_kinds:
-            self.custom_prepare_kinds[kind_name](body)
+        if kind.name in self.custom_prepare_kinds:
+            self.custom_prepare_kinds[kind.name](body)
 
         return body
 
@@ -258,12 +260,14 @@ class JobRunner:
         self,
         timeout: int = 60 * 5,
         watcher_start_timeout: int = 10,
+        on_kube_api_event: callable = None,
     ):
         """Execute the job
 
         Args:
             timeout (int, optional): Execution timeout. Defaults to 60*5.
             watcher_start_timeout (int, optional): The timeout to start watching the namespaces. Defaults to 10.
+            on_kube_api_event(callable, optional): Handle kube api events.
 
         Returns:
             KubeResourceState: The main resource (resources[0]) final state.
@@ -331,6 +335,16 @@ class JobRunner:
             label_selector=self.job_label_selector,
         )
 
+        def handle_kube_api_event(event: KubeLogApiEvent):
+            self.log(f"{event.line.get_context_header()} KubeApiEvent {event.name} of {len(event.value)} [chars]")
+            if on_kube_api_event:
+                on_kube_api_event(event)
+
+        watcher.on(
+            watcher.pod_log_api_event_name,
+            handle_kube_api_event,
+        )
+
         # start the watcher
         if self.show_watcher_logs:
             watcher.pipe_to_logger(self.logger)
@@ -434,5 +448,5 @@ class JobRunner:
         self.log("Client stopped, execution aborted.")
 
 
-JobRunner.register_custom_prepare_kind("pod", JobRunner.custom_prepare_pod_kind)
-JobRunner.register_custom_prepare_kind("job", JobRunner.custom_prepare_job_kind)
+JobRunner.register_custom_prepare_kind("Pod", JobRunner.custom_prepare_pod_kind)
+JobRunner.register_custom_prepare_kind("Job", JobRunner.custom_prepare_job_kind)
