@@ -2,15 +2,6 @@ import jinja2
 import json
 from typing import List, Union
 from airflow.utils.decorators import apply_defaults
-from airflow import version
-
-AIRFLOW_MAJOR_VERSION = int(version.version.split(".")[0])
-
-if AIRFLOW_MAJOR_VERSION > 1:
-    from airflow.models import BaseOperator
-else:
-    from airflow.operators import BaseOperator
-
 from airflow_kubernetes_job_operator.kube_api import KubeResourceState, KubeLogApiEvent
 from airflow_kubernetes_job_operator.utils import (
     to_kubernetes_valid_name,
@@ -25,9 +16,29 @@ from airflow_kubernetes_job_operator.config import (
     DEFAULT_DELETE_POLICY,
     DEFAULT_TASK_STARTUP_TIMEOUT,
     DEFAULT_KUBERNETES_MAX_RESOURCE_NAME_LENGTH,
+    IS_AIRFLOW_ONE,
+    AIRFLOW_MAJOR_VERSION,
 )
 
 
+# Base class is determined by the airflow version.
+if IS_AIRFLOW_ONE:
+    from airflow.operators import BaseOperator
+
+    class KubernetesJobOperatorDefaultsBase(BaseOperator):
+        @apply_defaults
+        def __init__(self, **kwargs) -> None:
+            super().__init__(**kwargs)
+
+
+else:
+    from airflow.models import BaseOperator
+
+    class KubernetesJobOperatorDefaultsBase(BaseOperator):
+        def __init__(self, **kwargs) -> None:
+            super().__init__(**kwargs)
+
+
 def xcom_value_parser(value: str) -> dict:
     value = value.strip()
     try:
@@ -40,12 +51,6 @@ def xcom_value_parser(value: str) -> dict:
     return value
 
 
-class KubernetesJobOperatorDefaultsBase(BaseOperator):
-    @apply_defaults
-    def __init__(self, **kwargs) -> None:
-        super().__init__(**kwargs)
-
-
 class KubernetesJobOperator(KubernetesJobOperatorDefaultsBase):
     autogenerate_job_id_from_task_id: bool = True
     resolve_relative_path_callstack_offset: int = 0
